#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <signal.h>

#include "imapfilter.h"


extern char *home;

static char *lockfile = NULL;	/* Lock file to stop new imapfilter
				   processes. */


/*
 * Create lockfile with the PID of imapfilter as its contents.
 */
void lockfile_create(void)
{
    FILE *fd;

    create_file(lockfile, S_IRUSR | S_IWUSR);

    fd = fopen(lockfile, "w");
    if (!fd)
	fatal(ERROR_FILE_OPEN, "imapfilter: opening lockfile; %s\n",
	      strerror(errno));
    fprintf(fd, "%s\n", ultostr(getpid(), 10));
    fclose(fd);
}


/*
 * Die if another instance of imapfilter is running.
 */
void lockfile_check(void)
{
    pid_t n;

    if ((n = lockfile_pid())) {
	fprintf(stderr,
	      "imapfilter: another imapfilter is running with pid %d\n", n);
	exit(ERROR_LOCKFILE);
    }
}


/*
 * Check if lockfile exists and if so, return the PID of the other
 * imapfilter running.
 */
pid_t lockfile_pid(void)
{
    char lockf[PATH_MAX];
    char line[LINE_MAX];
    FILE *fd;
    pid_t n = 0;

    snprintf(lockf, PATH_MAX, "%s/%s", home, ".imapfilter/lock");
    lockfile = xstrdup(lockf);

    if (exists_file(lockfile)) {
	fd = fopen(lockfile, "r");
	if (!fd)
	    fatal(ERROR_FILE_OPEN, "imapfilter: opening lockfile; %s\n",
		  strerror(errno));
	if (fgets(line, LINE_MAX - 1, fd)) {
	    errno = 0;
	    n = strtoul(line, NULL, 10);
	    if (errno)
		n = 0;
	}
	fclose(fd);
    }
    return n;
}


/*
 * Unlink the lockfile.
 */
int lockfile_remove(void)
{
    if (!lockfile)
	return 0;

    if (unlink(lockfile)) {
	error("imapfilter: removing lockfile; %s\n", strerror(errno));
	return ERROR_FILE_OPEN;
    }
    xfree(lockfile);

    return 0;
}


/*
 * Kill the other instance of imapfilter (if one is running).
 */
void kill_imapfilter(void)
{
    pid_t n;

    n = lockfile_pid();

    if (n > 0) {
	if (kill(n, SIGTERM))
	    fprintf(stderr, "imapfilter: killing process with pid %d; %s\n",
		    n, strerror(errno));
	exit(0);
    } else {
	fprintf(stderr, "imapfilter: no other imapfilter is running\n");
	exit(ERROR_UNDEFINED);
    }
}
